@@ -2,11 +2,11 @@
 Definitions of various hard coded constants.
 
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/python/Pyro
+irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
 """
 
 # Pyro version
-VERSION = "4.6"
+VERSION = "4.7"
 
 # standard object name for the Daemon object
 DAEMON_NAME = "Pyro.Daemon"
