@@ -2,19 +2,23 @@
 Core logic (uri, daemon, proxy stuff).
 
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/python/Pyro
+irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
 """
 
 from __future__ import with_statement
 import re, struct, sys, time, os
 import logging, uuid
 import hashlib, hmac
+try:
+    import copyreg
+except ImportError:
+    import copy_reg as copyreg
 from Pyro4 import constants, threadutil, util, socketutil, errors
 from Pyro4.socketserver.threadpoolserver import SocketServer_Threadpool
 from Pyro4.socketserver.multiplexserver import SocketServer_Select, SocketServer_Poll
 import Pyro4
 
-__all__=["URI", "Proxy", "Daemon", "callback"]
+__all__=["URI", "Proxy", "Daemon", "callback", "batch", "async"]
 
 if sys.version_info>(3, 0):
     basestring=str
@@ -131,6 +135,7 @@ class _RemoteMethod(object):
     def __call__(self, *args, **kwargs):
         return self.__send(self.__name, args, kwargs)
 
+
 def _check_hmac():
     if Pyro4.config.HMAC_KEY is None or len(Pyro4.config.HMAC_KEY)==0:
         import warnings
@@ -238,6 +243,8 @@ class Proxy(object):
                     self.__pyroCheckSequence(seq)
                     data=self._pyroSerializer.deserialize(data, compressed=flags & MessageFactory.FLAGS_COMPRESSED)
                     if flags & MessageFactory.FLAGS_EXCEPTION:
+                        if sys.platform=="cli":
+                            fixIronPythonExceptionForPickle(data, False) # extract
                         raise data
                     else:
                         return data
@@ -329,6 +336,9 @@ class Proxy(object):
             flags|=MessageFactory.FLAGS_ONEWAY
         return self.__pyroInvoke("<batch>", calls, None, flags)
 
+    def _pyroAsync(self, callback=None):
+        return _AsyncProxy(self.__pyroInvoke, callback)
+
 
 class _BatchedRemoteMethod(object):
     """method call abstraction that is used with batched calls"""
@@ -363,11 +373,84 @@ class _BatchProxy(object):
             else:
                 yield result   # it is a regular result object, yield that and continue.
 
-    def __call__(self, oneway=False):
-        results=self.__batchmethod(self.__calls, oneway)
+    def __call__(self, oneway=False, async=False, callback=None):
+        if oneway and async:
+            raise ValueError("async oneway calls make no sense")
+        if async:
+            return _AsyncRemoteMethod(self.__call_async, callback, "<asyncbatch>")()
+        else:
+            results=self.__batchmethod(self.__calls, oneway)
+            self.__calls=[]   # clear for re-use
+            if not oneway:
+                return self.__resultsgenerator(results)
+
+    def __call_async(self,name,args,kwargs):
+        # ignore all parameters, we just need to execute the batch
+        results=self.__batchmethod(self.__calls)
         self.__calls=[]   # clear for re-use
-        if not oneway:
-            return self.__resultsgenerator(results)
+        return self.__resultsgenerator(results)
+
+
+class _AsyncProxy(object):
+    def __init__(self, invokemethod, callback):
+        self.__invokemethod=invokemethod
+        self.__callback=callback
+
+    def __getattr__(self, name):
+        return _AsyncRemoteMethod(self.__invokemethod, self.__callback, name)
+
+
+class _AsyncRemoteMethod(object):
+    """async method call abstraction (call will run in a background thread)"""
+    def __init__(self, send, callback, name):
+        self.__send = send
+        self.__name = name
+        self.__callback = callback
+
+    def __getattr__(self, name):
+        return _AsyncRemoteMethod(self.__send, self.__callback, "%s.%s" % (self.__name, name))
+
+    def __call__(self, *args, **kwargs):
+        asyncresult=_AsyncResult()
+        thread=threadutil.Thread(target=self.__asynccall, args=(asyncresult, self.__callback, args, kwargs))
+        thread.setDaemon(True)
+        thread.start()
+        return asyncresult
+
+    def __asynccall(self, asyncresult, callback, args, kwargs):
+        try:
+            value = self.__send(self.__name, args, kwargs)
+            if callback:
+                callback(value)  # only provide the result value through the callback
+            else:
+                asyncresult.value=value
+        except:
+            log.warn("exception occurred in async call, ignored")
+            asyncresult.value=_ExceptionWrapper(sys.exc_info()[1])
+
+
+class _AsyncResult(object):
+    def __init__(self):
+        self.__ready=threadutil.Event()
+    def ready(self, timeout=None):
+        if timeout is None:
+            return self.__ready.isSet()
+        else:
+            self.__ready.wait(timeout)
+            if self.__ready.isSet():
+                return True
+            else:
+                raise Pyro4.errors.AsyncResultTimeout("async result didn't arrive in time")
+    def get_value(self):
+        self.__ready.wait()
+        if isinstance(self.__value, _ExceptionWrapper):
+            self.__value.raiseIt()
+        else:
+            return self.__value
+    def set_value(self, value):
+        self.__value=value
+        self.__ready.set()
+    value=property(get_value, set_value)
 
 
 class _ExceptionWrapper(object):
@@ -379,9 +462,20 @@ class _ExceptionWrapper(object):
         self.exception=exception
 
     def raiseIt(self):
+        if sys.platform=="cli":
+            fixIronPythonExceptionForPickle(self.exception, False) # extract attributes
         raise self.exception
 
 
+def batch(proxy):
+    """convenience method to get a batch proxy"""
+    return proxy._pyroBatch()
+
+def async(proxy, callback=None):
+    """convenience method to get an async proxy"""
+    return proxy._pyroAsync(callback)
+
+
 class MessageFactory(object):
     """internal helper class to construct Pyro protocol messages"""
     headerFmt = '!4sHHHHiH20s'    # header (id, version, msgtype, flags, sequencenumber, dataLen, checksum, hmac)
@@ -460,6 +554,20 @@ class MessageFactory(object):
         return msgType, flags, seq, databytes
 
 
+def pyroObjectSerializer(self):
+    """reduce function that automatically replaces Pyro objects by a Proxy"""
+    daemon=getattr(self,"_pyroDaemon",None)
+    if daemon:
+        return Pyro4.core.Proxy, (daemon.uriFor(self),)
+    else:
+        return self.__reduce__()
+
+
+def defaultObjectSerializer(self):
+    """reduce function that uses the default implementation"""
+    return self.__reduce__()
+
+
 class DaemonObject(object):
     """The part of the daemon that is exposed as a Pyro object."""
     def __init__(self, daemon):
@@ -513,6 +621,35 @@ class Daemon(object):
     def sockets(self):
         return self.transportServer.sockets
 
+    @staticmethod
+    def serveSimple(objects, daemon=None, ns=True, verbose=True):
+        """
+        Very basic method to fire up a daemon (or supply one yourself).
+        objects is a dict containing objects to register as keys, and
+        their names (or None) as values. If ns is true they will be registered
+        in the naming server as well, otherwise they just stay local.
+        """
+        if not daemon:
+            daemon=Daemon()
+        with daemon:
+            if ns:
+                ns=Pyro4.naming.locateNS()
+            for obj, name in objects.items():
+                if ns:
+                    localname=None   # name is used for the name server
+                else:
+                    localname=name   # no name server, use name in daemon
+                uri=daemon.register(obj, localname)
+                if verbose:
+                    print("Object {0}:\n    uri = {1}".format(repr(obj), uri))
+                if name and ns:
+                    ns.register(name, uri)
+                    if verbose:
+                        print("    name = {0}".format(name))
+            if verbose:
+                print("Pyro daemon running.")
+            daemon.requestLoop()
+
     def requestLoop(self, loopCondition=lambda: True):
         """
         Goes in a loop to service incoming requests, until someone breaks this
@@ -586,6 +723,8 @@ class Daemon(object):
                             xt,xv=sys.exc_info()[0:2]
                             log.debug("Exception occurred while handling batched request: %s", xv)
                             xv._pyroTraceback=util.formatTraceback(detailed=Pyro4.config.DETAILED_TRACEBACK)
+                            if sys.platform=="cli":
+                                fixIronPythonExceptionForPickle(xv, True)  # piggyback attributes
                             data.append(_ExceptionWrapper(xv))
                             break   # stop processing the rest of the batch
                         else:
@@ -630,6 +769,8 @@ class Daemon(object):
     def sendExceptionResponse(self, connection, seq, exc_value, tbinfo):
         """send an exception back including the local traceback info"""
         exc_value._pyroTraceback=tbinfo
+        if sys.platform=="cli":
+            fixIronPythonExceptionForPickle(exc_value, True)  # piggyback attributes
         data, _=self.serializer.serialize(exc_value)
         msg=MessageFactory.createMessage(MessageFactory.MSG_RESULT, data, MessageFactory.FLAGS_EXCEPTION, seq)
         del data
@@ -650,8 +791,16 @@ class Daemon(object):
             raise errors.DaemonError("object already has a Pyro id")
         if objectId in self.objectsById:
             raise errors.DaemonError("object already registered")
+        # set some pyro attributes
         obj._pyroId=objectId
         obj._pyroDaemon=self
+        if Pyro4.config.AUTOPROXY:
+            # register a custom serializer for the type to automatically return proxies
+            try:
+                copyreg.pickle(type(obj),pyroObjectSerializer)
+            except TypeError:
+                pass
+        # register the object in the mapping
         self.objectsById[obj._pyroId]=obj
         return self.uriFor(objectId)
 
@@ -676,6 +825,12 @@ class Daemon(object):
             if objectOrId is not None:
                 del objectOrId._pyroId
                 del objectOrId._pyroDaemon
+                if Pyro4.config.AUTOPROXY:
+                    # remove the custom type serializer
+                    try:
+                        copyreg.pickle(type(objectOrId),defaultObjectSerializer)
+                    except TypeError:
+                        pass
 
     def uriFor(self, objectOrId=None):
         """
@@ -709,6 +864,30 @@ class Daemon(object):
     def __exit__(self, exc_type, exc_value, traceback):
         self.close()
 
+    def __getstate__(self):
+        return {}   # a little hack to make it possible to serialize Pyro objects.
+
+
+class __IronPythonExceptionArgs(object):
+    def __init__(self,data):
+        self.data=data
+
+def fixIronPythonExceptionForPickle(exceptionObject, addAttributes):
+    """function to hack around a bug in IronPython where it doesn't pickle
+    exception attributes. We piggyback them into the exception's args."""
+    if hasattr(exceptionObject, "args"):
+        if addAttributes:
+            # piggyback the attributes on the exception args instead.
+            exceptionObject.args+=(__IronPythonExceptionArgs(vars(exceptionObject)),)
+        else:
+            # check if there is a piggybacked object in the args
+            # if there is, extract the exception attributes from it.
+            if len(exceptionObject.args) > 0:
+                piggyback = exceptionObject.args[-1]
+                if isinstance(piggyback, __IronPythonExceptionArgs):
+                    exceptionObject.args = exceptionObject.args[:-1]
+                    exceptionObject.__dict__.update(piggyback.data)
+
 
 # decorators
 
@@ -720,3 +899,4 @@ def callback(object):
     """
     object._pyroCallback=True
     return object
+
