@@ -2,7 +2,7 @@
 Exception definitions.
 
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/python/Pyro
+irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
 """
 
 
@@ -35,4 +35,8 @@ class DaemonError(PyroError):
 
 
 class SecurityError(PyroError):
-    pass
\ No newline at end of file
+    pass
+
+
+class AsyncResultTimeout(PyroError):
+    pass
