@@ -2,7 +2,7 @@
 Name server control tool.
 
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/python/Pyro
+irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
 """
 
 import sys
@@ -35,7 +35,7 @@ def handleCommand(nameserver, options, args):
         printListResult(nameserver.list(regex=args[1]), "- regex '%s'" % args[1])
 
     def cmd_register():
-        nameserver.register(args[1], args[2])
+        nameserver.register(args[1], args[2], safe=True)
         print("Registered %s" % args[1])
 
     def cmd_remove():
@@ -65,7 +65,7 @@ def handleCommand(nameserver, options, args):
         commands[args[0]]()
     except Exception:
         x=sys.exc_info()[1]
-        print("Error: %s" % x)
+        print("Error: %s" % repr(x))
 
 
 def main(args):
