@@ -2,7 +2,7 @@
 Socket server based on socket multiplexing. Doesn't use threads.
 
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/python/Pyro
+irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
 """
 
 import socket, select, sys, logging
