@@ -4,7 +4,7 @@ Socket server based on a worker thread pool. Doesn't use select.
 Uses a single worker thread per client connection.
 
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/python/Pyro
+irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
 """
 
 from __future__ import with_statement
