@@ -2,7 +2,7 @@
 Echo server for test purposes.
 
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/python/Pyro
+irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
 """
 
 import sys, os, time
@@ -17,17 +17,17 @@ class EchoServer(object):
 
     def echo(self, args):
         if self.verbose:
-            print ("%s - echo: %s" % (time.asctime(), args))
+            print("%s - echo: %s" % (time.asctime(), args))
         return args
 
     def error(self):
         if self.verbose:
-            print ("%s - error: generating exception" % time.asctime())
+            print("%s - error: generating exception" % time.asctime())
         return 1//0   # division by zero error
 
     def shutdown(self):
         if self.verbose:
-            print ("%s - shutting down" % time.asctime())
+            print("%s - shutting down" % time.asctime())
         self.must_shutdown=True
 
 
@@ -60,22 +60,22 @@ def main(args, returnWithoutLooping=False):
     parser.add_option("-N","--nameserver", action="store_true", default=False, help="also start a nameserver")
     parser.add_option("-v","--verbose", action="store_true", default=False, help="verbose output")
     parser.add_option("-q","--quiet", action="store_true", default=False, help="don't output anything")
+    parser.add_option("-k","--key", help="the HMAC key to use")
     options,args = parser.parse_args(args)
 
     if options.verbose:
         options.quiet=False
     if not options.quiet:
-        print ("Starting Pyro's built-in test echo server.")
+        print("Starting Pyro's built-in test echo server.")
     if os.name!="java":
         Pyro4.config.SERVERTYPE="multiplex"
 
-    if not Pyro4.config.HMAC_KEY:
-        if sys.version_info<(3,0):
-            Pyro4.config.HMAC_KEY="testkey"
-        else:
-            Pyro4.config.HMAC_KEY=bytes("testkey","utf-8")
-        if not options.quiet:
-            print("HMAC_KEY set to built-in default: %s" % Pyro4.config.HMAC_KEY)
+    hmac=options.key
+    if hmac and sys.version_info>=(3,0):
+        hmac=bytes(hmac,"utf-8")
+    Pyro4.config.HMAC_KEY=hmac or Pyro4.config.HMAC_KEY
+    if not options.quiet and Pyro4.config.HMAC_KEY:
+        print("HMAC_KEY set to: %s" % Pyro4.config.HMAC_KEY)
 
     nameserver=None
     if options.nameserver:
@@ -92,17 +92,16 @@ def main(args, returnWithoutLooping=False):
         if nameserver is not None:
             host,port=nameserver.uri.host, nameserver.uri.port
         ns=naming.locateNS(host,port)
-        ns.remove(objectName)
         ns.register(objectName, uri)
         if options.verbose:
-            print ("using name server at %s" % ns._pyroUri)
+            print("using name server at %s" % ns._pyroUri)
     else:
         if options.verbose:
-            print ("not using a name server.")
+            print("not using a name server.")
     if not options.quiet:
-        print ("object name: %s" % objectName)
-        print ("echo uri: %s" % uri)
-        print ("echoserver running.")
+        print("object name: %s" % objectName)
+        print("echo uri: %s" % uri)
+        print("echoserver running.")
 
     if returnWithoutLooping:
         return d,echo,uri        # for unit testing
