@@ -3,7 +3,7 @@ Threading abstraction which allows for threading2 use with a
 transparent fallback to threading when not available.
 
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/python/Pyro
+irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
 """
 
 from Pyro4 import config
