@@ -2,14 +2,14 @@
 Tests for the core logic.
 
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/python/Pyro
+irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
 """
 
 from __future__ import with_statement
 import unittest
 import copy
 import logging
-import os, sys
+import os, sys, time
 import Pyro4.configuration
 import Pyro4.core
 import Pyro4.errors
@@ -436,6 +436,16 @@ class RemoteMethodTests(unittest.TestCase):
             else:
                 return self.result
 
+    class AsyncProxyMock(object):
+        def _pyroAsync(self, callback=None):
+            return Pyro4.core._AsyncProxy(self.pyroInvoke, callback)
+        def pyroInvoke(self, methodname, vargs, kwargs, flags=0):
+            if methodname=="pause_and_divide":
+                time.sleep(vargs[0])
+                return vargs[1]//vargs[2]
+            else:
+                raise NotImplementedError(methodname)
+
     def testRemoteMethod(self):
         class ProxyMock(object):
             def invoke(self, name, args, kwargs):
@@ -454,7 +464,7 @@ class RemoteMethodTests(unittest.TestCase):
 
     def testBatchMethod(self):
         proxy=self.BatchProxyMock()
-        batch=proxy._pyroBatch()
+        batch=Pyro4.batch(proxy)
         self.assertEqual(None, batch.foo(42))
         self.assertEqual(None, batch.bar("abc"))
         self.assertEqual(None, batch.baz(42,"abc",arg=999))
@@ -473,7 +483,7 @@ class RemoteMethodTests(unittest.TestCase):
 
     def testBatchMethodOneway(self):
         proxy=self.BatchProxyMock()
-        batch=proxy._pyroBatch()
+        batch=Pyro4.batch(proxy)
         self.assertEqual(None, batch.foo(42))
         self.assertEqual(None, batch.bar("abc"))
         self.assertEqual(None, batch.baz(42,"abc",arg=999))
@@ -485,7 +495,7 @@ class RemoteMethodTests(unittest.TestCase):
 
     def testBatchMethodReuse(self):
         proxy=self.BatchProxyMock()
-        batch=proxy._pyroBatch()
+        batch=Pyro4.batch(proxy)
         batch.foo(1)
         batch.foo(2)
         results=batch()
@@ -498,6 +508,58 @@ class RemoteMethodTests(unittest.TestCase):
         results=batch()
         self.assertEqual(0, len(list(results)))
 
+    def testAsyncMethod(self):
+        proxy=self.AsyncProxyMock()
+        async=Pyro4.async(proxy)
+        begin=time.time()
+        result=async.pause_and_divide(1,10,2)  # returns immediately
+        duration=time.time()-begin
+        self.assertTrue(duration<0.1)
+        self.assertFalse(result.ready())
+        _=result.value
+        self.assertTrue(result.ready())
+
+    def testAsyncCallbackMethod(self):
+        def asyncCallback(value):
+            self.assertEqual(5,value)
+
+        proxy=self.AsyncProxyMock()
+        async=Pyro4.async(proxy, callback=asyncCallback)
+        result=async.pause_and_divide(1,10,2)  # returns immediately
+        self.assertFalse(result.ready())
+        # can't use result.value here because we're using a callback.
+
+    def testAsyncMethodTimeout(self):
+        proxy=self.AsyncProxyMock()
+        async=Pyro4.async(proxy)
+        result=async.pause_and_divide(1,10,2)  # returns immediately
+        self.assertFalse(result.ready())
+        self.assertRaises(Pyro4.errors.AsyncResultTimeout, result.ready, 0.5)  # won't be ready after 0.5 sec
+        self.assertTrue(result.ready(2))  # will be ready within 2 seconds
+        self.assertEqual(5,result.value)
+
+
+class TestSimpleServe(unittest.TestCase):
+    class DaemonMock(object):
+        def __init__(self):
+            self.objects={}
+        def register(self, object, name):
+            self.objects[object]=name
+        def __enter__(self):
+            pass
+        def __exit__(self, *args):
+            pass
+        def requestLoop(self, *args):
+            pass
+
+    def testSimpleServe(self):
+        d=TestSimpleServe.DaemonMock()
+        o1=Thing(1)
+        o2=Thing(2)
+        objects={ o1: "test.o1", o2: None }
+        Pyro4.core.Daemon.serveSimple(objects,daemon=d, ns=False, verbose=False)
+        self.assertEqual( {o1: "test.o1", o2: None}, d.objects)
+
 
 if __name__ == "__main__":
     #import sys;sys.argv = ['', 'Test.testName']
