@@ -2,7 +2,7 @@
 Tests for the name server (offline/basic logic).
 
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/python/Pyro
+irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
 """
 
 from __future__ import with_statement
@@ -25,13 +25,18 @@ class OfflineNameServerTests(unittest.TestCase):
     def testRegister(self):
         ns=Pyro4.naming.NameServer()
         ns.ping()
-        ns.register("test.object1","PYRO:111111@host.com:4444")
+        ns.register("test.object1","PYRO:000000@host.com:4444")
         ns.register("test.object2","PYRO:222222@host.com:4444")
         ns.register("test.object3","PYRO:333333@host.com:4444")
+        self.assertEqual("PYRO:000000@host.com:4444",str(ns.lookup("test.object1")))
+        ns.register("test.object1","PYRO:111111@host.com:4444")  # registering again should be ok by default
+        self.assertEqual("PYRO:111111@host.com:4444",str(ns.lookup("test.object1")), "should be new uri")
         ns.register("test.sub.objectA",Pyro4.core.URI("PYRO:AAAAAA@host.com:4444"))
         ns.register("test.sub.objectB",Pyro4.core.URI("PYRO:BBBBBB@host.com:4444"))
 
-        self.assertRaises(NamingError, ns.register, "test.object1", "PYRO:X@Y:5555")
+        # if safe=True, a registration of an existing name shoould give a NamingError
+        self.assertRaises(NamingError, ns.register, "test.object1", "PYRO:X@Y:5555", safe=True)
+
         self.assertRaises(TypeError, ns.register, None, None)
         self.assertRaises(TypeError, ns.register, 4444, 4444)
         self.assertRaises(TypeError, ns.register, "test.wrongtype", 4444)
@@ -220,7 +225,7 @@ class OfflineNameServerTests(unittest.TestCase):
             sys.stderr=StringIO()
             ns=Pyro4.naming.NameServer()
             Pyro4.nsc.handleCommand(ns, None, ["foo"])
-            self.assertTrue(sys.stdout.getvalue().endswith("Error: 'foo'\n"))
+            self.assertTrue(sys.stdout.getvalue().endswith("Error: KeyError('foo',)\n"))
             Pyro4.nsc.handleCommand(ns, None, ["ping"])
             self.assertTrue(sys.stdout.getvalue().endswith("ping ok.\n"))
             Pyro4.nsc.handleCommand(ns, None, ["list"])
