@@ -2,7 +2,7 @@
 Tests for a running Pyro server, without timeouts.
 
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/python/Pyro
+irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
 """
 
 from __future__ import with_statement
@@ -10,7 +10,7 @@ import unittest
 import Pyro4.core
 import Pyro4.errors
 import Pyro4.util
-import time, os, sys
+import time, os, sys, platform
 from Pyro4 import threadutil
 from testsupport import *
 
@@ -26,6 +26,8 @@ class MyThing(object):
         return x//y
     def ping(self):
         pass
+    def echo(self, obj):
+        return obj
     def delay(self, delay):
         time.sleep(delay)
         return "slept %d seconds" % delay
@@ -114,15 +116,20 @@ class ServerTestsSingle(unittest.TestCase):
                 pass
             x=p.getDict()
             self.assertEqual({"number":42}, x)
-            # also test some argument type things
+
+    def testSomeArgumentTypes(self):
+        with Pyro4.core.Proxy(self.objectUri) as p:
             self.assertEqual((1,(),{}), p.testargs(1))
             self.assertEqual((1,(2,3),{'a':4}), p.testargs(1,2,3,a=4))
             self.assertEqual((1,(),{'a':2}), p.testargs(1, **{'a':2}))
             if sys.version_info>=(2,6,5):
                 # python 2.6.5 and later support unicode keyword args
-                result=p.testargs(1, **{unichr(0x20ac):2})
-                key=list(result[2].keys())[0]
-                self.assertTrue(key==unichr(0x20ac))
+                self.assertEqual((1,(),{unichr(65):2}), p.testargs(1, **{unichr(65):2}))
+                if platform.python_implementation()!="PyPy":
+                    # PyPy doesn't accept unicode kwargs that cannot be encoded to ascii, see https://bugs.pypy.org/issue751
+                    result=p.testargs(1, **{unichr(0x20ac):2})
+                    key=list(result[2].keys())[0]
+                    self.assertTrue(key==unichr(0x20ac))
 
     def testDottedNames(self):
         try:
@@ -143,7 +150,7 @@ class ServerTestsSingle(unittest.TestCase):
 
     def testBatchProxy(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
-            batch=p._pyroBatch()
+            batch=Pyro4.batch(p)
             self.assertEqual(None,batch.multiply(7,6))
             self.assertEqual(None,batch.divide(999,3))
             self.assertEqual(None,batch.ping())
@@ -156,6 +163,71 @@ class ServerTestsSingle(unittest.TestCase):
             self.assertRaises(ZeroDivisionError, next, results)     # 999//0 should raise this error
             self.assertRaises(StopIteration, next, results)     # no more results should be available after the error
 
+    def testAsyncProxy(self):
+        with Pyro4.core.Proxy(self.objectUri) as p:
+            async=Pyro4.async(p)
+            begin=time.time()
+            result=async.delayAndId(1,42)
+            duration=time.time()-begin
+            self.assertTrue(duration<0.1)
+            self.assertFalse(result.ready())
+            self.assertRaises(Pyro4.errors.AsyncResultTimeout, result.ready, 0.5)
+            self.assertEqual("slept for 42",result.value)
+            self.assertTrue(result.ready())
+
+    def testPyroTracebackNormal(self):
+        with Pyro4.core.Proxy(self.objectUri) as p:
+            try:
+                p.divide(999,0)  # force error here
+                self.fail("expected error")
+            except ZeroDivisionError:
+                # going to check if the magic pyro traceback attribute is available for batch methods too
+                tb="".join(Pyro4.util.getPyroTraceback())
+                self.assertTrue("Remote traceback:" in tb)  # validate if remote tb is present
+                self.assertTrue("ZeroDivisionError" in tb)  # the error
+                self.assertTrue("return x//y" in tb)  # the statement
+
+    def testPyroTracebackBatch(self):
+        with Pyro4.core.Proxy(self.objectUri) as p:
+            batch=Pyro4.batch(p)
+            self.assertEqual(None,batch.divide(999,0))      # force an exception here
+            results=batch()
+            try:
+                next(results)
+                self.fail("expected error")
+            except ZeroDivisionError:
+                # going to check if the magic pyro traceback attribute is available for batch methods too
+                tb="".join(Pyro4.util.getPyroTraceback())
+                self.assertTrue("Remote traceback:" in tb)  # validate if remote tb is present
+                self.assertTrue("ZeroDivisionError" in tb)  # the error
+                self.assertTrue("return x//y" in tb)  # the statement
+            self.assertRaises(StopIteration, next, results)     # no more results should be available after the error
+
+    def testAutoProxy(self):
+        obj=MyThing()
+        try:
+            with Pyro4.core.Proxy(self.objectUri) as p:
+                Pyro4.config.AUTOPROXY=True   # make sure autoproxying is enabled
+                result=p.echo(obj)
+                self.assertTrue(isinstance(result,MyThing))
+                self.daemon.register(obj)
+                result=p.echo(obj)
+                self.assertTrue(isinstance(result, Pyro4.core.Proxy),"serialized pyro object must be a proxy")
+                self.daemon.unregister(obj)
+                result=p.echo(obj)
+                self.assertTrue(isinstance(result,MyThing), "serialized object must be normal again")
+                Pyro4.config.AUTOPROXY=False   # try again without auto proxying
+                result=p.echo(obj)
+                self.assertTrue(isinstance(result,MyThing))
+                self.daemon.register(obj)
+                result=p.echo(obj)
+                self.assertTrue(isinstance(result,MyThing), "with autoproxy off the object should be an instance of the class")
+                self.daemon.unregister(obj)
+                result=p.echo(obj)
+                self.assertTrue(isinstance(result,MyThing), "serialized object must be normal again")
+        finally:
+            Pyro4.config.AUTOPROXY=True
+
 
 class ServerTestsThreadNoTimeout(unittest.TestCase):
     SERVERTYPE="thread"
@@ -316,7 +388,7 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
                 et,ev,tb=sys.exc_info()
                 self.assertEqual(ZeroDivisionError, et)
                 pyrotb="".join(Pyro4.util.getPyroTraceback(et,ev,tb))
-                self.assertTrue("Remote traceback" in pyrotb)    # fails on ironpython...
+                self.assertTrue("Remote traceback" in pyrotb)
                 self.assertTrue("ZeroDivisionError" in pyrotb)
                 del tb
 
@@ -335,7 +407,7 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
             if sys.platform!="cli":
                 self.assertAlmostEqual(0.1, duration, places=1)
             else:
-                # ironpython's time is wonky
+                # ironpython's time is weird
                 self.assertTrue(0.0<duration<0.7)
 
     def testTimeoutConnect(self):
